@@ -538,7 +538,7 @@ zebra_config_write (struct vty *vty)
 }
 
 /* Zebra node structure. */
-struct cmd_node zebra_node =
+static struct cmd_node zebra_node =
 {
   ZEBRA_NODE,
   "%s(config-router)# ",
