@@ -87,12 +87,6 @@ rtadv_new (void)
   return XCALLOC (MTYPE_TMP, sizeof (struct rtadv));
 }
 
-static void
-rtadv_free (struct rtadv *rtadv)
-{
-  XFREE (MTYPE_TMP, rtadv);
-}
-
 static int
 rtadv_recv_packet (int sock, u_char *buf, int buflen,
 		   struct sockaddr_in6 *from, unsigned int *ifindex,
@@ -138,7 +132,10 @@ rtadv_recv_packet (int sock, u_char *buf, int buflen,
       /* Incoming packet's hop limit. */
       if (cmsgptr->cmsg_level == IPPROTO_IPV6 &&
 	  cmsgptr->cmsg_type == IPV6_HOPLIMIT)
-	*hoplimit = *((int *) CMSG_DATA (cmsgptr));
+	{
+	  int *hoptr = (int *) CMSG_DATA (cmsgptr);
+	  *hoplimit = *hoptr;
+	}
     }
   return ret;
 }
@@ -292,19 +289,33 @@ rtadv_send_packet (int sock, struct interface *ifp)
   if (sdl != NULL && sdl->sdl_alen != 0)
     {
       buf[len++] = ND_OPT_SOURCE_LINKADDR;
-      buf[len++] = (sdl->sdl_alen + 2) >> 3;
+
+      /* Option length should be rounded up to next octet if
+         the link address does not end on an octet boundary. */
+      buf[len++] = (sdl->sdl_alen + 9) >> 3;
 
       memcpy (buf + len, LLADDR (sdl), sdl->sdl_alen);
       len += sdl->sdl_alen;
+
+      /* Pad option to end on an octet boundary. */
+      memset (buf + len, 0, -(sdl->sdl_alen + 2) & 0x7);
+      len += -(sdl->sdl_alen + 2) & 0x7;
     }
 #else
   if (ifp->hw_addr_len != 0)
     {
       buf[len++] = ND_OPT_SOURCE_LINKADDR;
-      buf[len++] = (ifp->hw_addr_len + 2) >> 3;
+
+      /* Option length should be rounded up to next octet if
+         the link address does not end on an octet boundary. */
+      buf[len++] = (ifp->hw_addr_len + 9) >> 3;
 
       memcpy (buf + len, ifp->hw_addr, ifp->hw_addr_len);
       len += ifp->hw_addr_len;
+
+      /* Pad option to end on an octet boundary. */
+      memset (buf + len, 0, -(ifp->hw_addr_len + 2) & 0x7);
+      len += -(ifp->hw_addr_len + 2) & 0x7;
     }
 #endif /* HAVE_STRUCT_SOCKADDR_DL */
 
@@ -453,7 +464,7 @@ rtadv_read (struct thread *thread)
   int len;
   u_char buf[RTADV_MSG_SIZE];
   struct sockaddr_in6 from;
-  unsigned int ifindex;
+  unsigned int ifindex = 0;
   int hoplimit = -1;
 
   sock = THREAD_FD (thread);
